import DTU from '../../utils/datetime'
import {
  FMT_HTML5_DATETIME_LOCAL, FMT_DATE_TIME
} from '../../utils/consts'

export const DTP_TYPES = {
  date: 'date',
  datetime: 'datetime-local',
  time: 'time',
  month: 'month',
  week: 'week'
}


export default {
  name: 'RuiDateTimePicker',
  model: {
    prop: 'modelValue',
    event: 'changed'
  },
  props: {
    modelValue: { type: String },
    type: { type: String, default: DTP_TYPES.datetime },
    min: { type: String },
    max: { type: String },
    width: { type: [Number, String], default: '' },
    customClass: { type: String, default: '' }
  },
  data() {
    return {
      value: '',
      timer: null
    }
  },
  watch: {
    modelValue: {
      handler(nv, ov) {
        if (nv !== ov) {
          this.$nextTick(() => {
            this.value = this.formatValue(nv)
          })
        }
      },
      immediate: true
    }
  },
  methods: {
    /* 事件处理 */
    hChanged(e) {
      if (this.timer) clearTimeout(this.timer)
      this.timer = setTimeout(() => {
        this.valueChanged(e)
      }, 500)
    },
    valueChanged(e) {
      switch (this.type) {
        case DTP_TYPES.date:
          this.$emit('changed', e.target.value)
          break
        case DTP_TYPES.datetime:
          this.$emit(
            'changed', DTU.format(DTU.parse(e.target.value, FMT_HTML5_DATETIME_LOCAL), FMT_DATE_TIME)
          )
          break
        case DTP_TYPES.time:
          this.$emit('changed', e.target.value)
          break
        case DTP_TYPES.month:
          this.$emit('changed', e.target.value)
          break
        case DTP_TYPES.week:
          this.$emit('changed', e.target.value)
          break
      }
    },
    formatValue(v) {
      let result = ''
      switch (this.type) {
        case DTP_TYPES.date:
          result = this.formatValue4Date(v)
          break
        case DTP_TYPES.datetime:
          result = this.formatValue4DateTime(v)
          break
        case DTP_TYPES.time:
          result = this.formatValue4Time(v)
          break
        case DTP_TYPES.month:
          result = this.formatValue4Month(v)
          break
        case DTP_TYPES.week:
          result = this.formatValue4Week(v)
          break
      }
      return result
    },
    formatValue4Date(v) {
      return v
    },
    formatValue4DateTime (v) {
      return DTU.format(
        DTU.parse(v, [
          'YYYY-MM-DD HH:mm:ss', 'YYYY-MM-DD HH:mm',
          'YYYY-MM-DD HH:mm:ss.SSS', 'YYYY-MM-DD'
        ])  
      )
    },
    formatValue4Time(v) {
      return v
    },
    formatValue4Month(v) {
      return v
    },
    formatValue4Week(v) {
      return v
    },
    calcMax() {
      let result = ''
      switch (this.type) {
        case DTP_TYPES.date:
          result = this.max || '9999-12-31'
          break
        case DTP_TYPES.datetime:
          result = this.max ? this.formatValue4DateTime(this.max) : '9999-12-31T23:59:59'
          break
        default:
          result = this.max
          break
      }
      return result
    },
    calcMin() {
      let result = ''
      switch (this.type) {
        case DTP_TYPES.datetime:
          result = this.min ? this.formatValue4DateTime(this.min) : ''
          break
        default:
          result = this.min
          break
      }
      return result
    }
  },
  render(h) {
    return h('input', {
      class: [this.customClass],
      domProps: {
        type: this.type,
        min: this.calcMin(),
        max: this.calcMax(),
        value: this.value
      },
      style: {
        width: this.width ? (typeof this.width === 'string' ? this.width : `${this.width}px`) : ''
      },
      on: {
        change: this.hChanged
      }
    })
  }
}


// export default {
//   name: 'RuiDateTimePicker',
//   model: {
//     prop: 'mValue',
//     event: 'mValueChanged'
//   },
//   props: {
//     name: { type: String },
//     mValue: { type: String },
//     type: { type: String, default: DTP_TYPES.date },
//     min: { type: String },
//     max: { type: String },
//     width: { type: [Number, String], default: '' },
//     customClass: { type: String, default: '' },
//     disabled: { type: Boolean, default: false },
//     readOnly: { type: Boolean, default: false }
//   },
//   data () {
//     return {
//       value: ''
//     }
//   },
//   watch: {
//     mValue: {
//       handler (newV) { this.value = this.formatValue(newV) }, immediate: true
//     }
//   },
//   computed: {
//     // minValue () {
//     //   return this.calcMin()
//     // },
//     // maxValue () {
//     //   return this.calcMax()
//     // }
//   },
//   methods: {

//     /* 事件处理 */
//     hChanged(e) {
//       if (this.timer) clearTimeout(this.timer)
//       this.timer = setTimeout(() => {
//         this.valueChanged(e)
//       }, 500)
//     },
//     valueChanged(e) {
//       switch (this.type) {
//         case DTP_TYPES.date:
//           this.$emit('changed', e.target.value)
//           break
//         case DTP_TYPES.datetime:
//           this.$emit(
//             'changed', DTU.format(DTU.parse(e.target.value, FMT_HTML5_DATETIME_LOCAL), FMT_DATE_TIME)
//           )
//           // moment(e.target.value, [moment.HTML5_FMT.DATETIME_LOCAL]).format('YYYY-MM-DD HH:mm'))
//           break
//         case DTP_TYPES.time:
//           this.$emit('changed', e.target.value)
//           break
//         case DTP_TYPES.month:
//           this.$emit('changed', e.target.value)
//           break
//         case DTP_TYPES.week:
//           this.$emit('changed', e.target.value)
//           break
//       }
//     },
//     formatValue(v) {
//       let result = ''
//       switch (this.type) {
//         case DTP_TYPES.date:
//           result = this.formatValue4Date(v)
//           break
//         case DTP_TYPES.datetime:
//           result = this.formatValue4DateTime(v)
//           break
//         case DTP_TYPES.time:
//           result = this.formatValue4Time(v)
//           break
//         case DTP_TYPES.month:
//           result = this.formatValue4Month(v)
//           break
//         case DTP_TYPES.week:
//           result = this.formatValue4Week(v)
//           break
//       }
//       return result
//     },
//     formatValue4Date(v) {
//       return v
//     },
//     formatValue4DateTime (v) {
//       return DTU.format(
//         DTU.parse(v, ['YYYY-MM-DD HH:mm:ss', 'YYYY-MM-DD HH:mm', 'YYYY-MM-DD HH:mm:ss.SSS', 'YYYY-MM-DD'])
//       )
//       // return moment(v, ['YYYY-MM-DD HH:mm:ss', 'YYYY-MM-DD HH:mm', 'YYYY-MM-DD HH:mm:ss.SSS', 'YYYY-MM-DD']).format(moment.HTML5_FMT.DATETIME_LOCAL)
//     },
//     formatValue4Time(v) {
//       return v
//     },
//     formatValue4Month(v) {
//       return v
//     },
//     formatValue4Week(v) {
//       return v
//     },
//     calcMax() {
//       let result = ''
//       switch (this.type) {
//         case DTP_TYPES.date:
//           result = this.max || '9999-12-31'
//           break
//         case DTP_TYPES.datetime:
//           result = this.max ? this.formatValue4DateTime(this.max) : '9999-12-31T23:59:59'
//           break
//         default:
//           result = this.max
//           break
//       }
//       return result
//     },
//     calcMin() {
//       console.log(JSON.stringify({ min: this.min, type: this.type, fmt: this.formatValue4DateTime(this.min) }))
//       let result = ''
//       switch (this.type) {
//         case DTP_TYPES.datetime:
//           result = this.min ? this.formatValue4DateTime(this.min) : ''
//           break
//         default:
//           result = this.min
//           break
//       }
//       return result
//     }
//   },
//   render (ce) {
//     return ce('input', {
//       class: [this.customClass],
//       domProps: {
//         type: this.type,
//         value: this.value,
//         min: this.calcMin(),
//         max: this.calcMax(),
//         disabled: this.disabled
//       },
//       style: {
//         width: this.width ? (typeof this.width === 'string' ? this.width : `${this.width}px`) : ''
//       },
//       on: {
//         change: this.hChanged
//       }
//     })
//   }
// }