import {isArray} from '../utils/iswhat'
import {
  REG_EXP_DATE_FMT_YEAR,
  REG_EXP_DATE_FMT_MONTH,
  REG_EXP_DATE_FMT_DAY,
  REG_EXP_DATE_FMT_HOUR,
  REG_EXP_DATE_FMT_MINUE,
  REG_EXP_DATE_FMT_SECOND,
  REG_EXP_DATE_FMT_MSECOND,
  REG_EXP_DATE_FMT_WEEK_YEAR,
  REG_EXP_DATE_FMT_WEEK_MONTH,
  FMT_DATE_TIME
} from './consts'

const TABLE_NEXT_MONDAY = [1, 7, 6, 5, 4, 3, 2]

const handleYear = (date, fmt) => {
  const [re] = fmt.match(REG_EXP_DATE_FMT_YEAR)
  let ret = fmt
  if (re) {
    ret = fmt.replace(re, `${takeYear(date)}`.substring(4 - re.length))
  }
  return ret
}

const handleOther = (fmt, val, regExp) => {
  const res = fmt.match(regExp)
  let ret = fmt
  if (res && res[0]) {
    const re = res[0]
    const tmp =  `00${val}`
    ret = fmt.replace(re, re.length === 1 ? val : tmp.substring(tmp.length - re.length))
  }
  return ret
}

const handleMonth = (date, fmt) => {
  const res = fmt.match(REG_EXP_DATE_FMT_WEEK_MONTH)
  const w = takeWeekOfMonth(date)
  if (w < 0 && res && res[0]) {
    return handleOther(fmt, takeMonth(calcLastMonth(date)), REG_EXP_DATE_FMT_MONTH)
  } else {
    return handleOther(fmt, takeMonth(date), REG_EXP_DATE_FMT_MONTH)
  }
}

const handleDay = (date, fmt) => {
  return handleOther(fmt, takeDayOfMonth(date), REG_EXP_DATE_FMT_DAY)
}

const handleHour = (date, fmt) => {
  return handleOther(fmt, takeHours(date), REG_EXP_DATE_FMT_HOUR)
}

const handleMinutes = (date, fmt) => {
  return handleOther(fmt, takeMinutes(date), REG_EXP_DATE_FMT_MINUE)
}

const handleSeconds = (date, fmt) => {
  return handleOther(fmt, takeSeconds(date), REG_EXP_DATE_FMT_SECOND)
}

const handleMilliseconds = (date, fmt) => {
  return handleOther(fmt, takeMilliseconds(date), REG_EXP_DATE_FMT_MSECOND)
}

const handleWeek = (date, fmt) => {
  return handleOther(fmt, takeWeekOfYear(date), REG_EXP_DATE_FMT_WEEK_YEAR)
}

const handleWeekMonth = (date, fmt) => {
  return handleOther(fmt, Math.abs(takeWeekOfMonth(date)), REG_EXP_DATE_FMT_WEEK_MONTH)
}

const handlers = [
  handleYear,
  handleMonth,
  handleDay,
  handleHour,
  handleMinutes,
  handleSeconds,
  handleMilliseconds,
  handleWeek,
  handleWeekMonth
]

export const takeYear = v => v.getFullYear()
export const takeMonth = v => v.getMonth() + 1
export const takeDayOfMonth = v => v.getDate()
export const takeHours = v => v.getHours()
export const takeMinutes = v => v.getMinutes()
export const takeSeconds = v => v.getSeconds()
export const takeMilliseconds = v => v.getMilliseconds()
export const takeWeekOfYear = v => {
  const table = TABLE_NEXT_MONDAY
  let fir = new Date(v.getFullYear(), 0, 1)
  const firw = fir.getDay()
  fir = calcDay(fir, table[firw])
  const d = Math.round((v.getTime() - fir.getTime()) / 86400000)
  return Math.ceil((d + (fir.getDay() +1 -1)) / 7)
}
export const takeWeekOfMonth = v => {
  const table = [-6, 0, -1, -2, -3, -4, -5]
  const vw = v.getDay()
  const w1 = calcDay(v, table[vw])
  const mv = v.getMonth()
  const m = w1.getMonth()
  let w
  for (let i = 0; i < 5; i++) {
    const tmp = calcDay(w1, -7 * (i + 1))
    if (m !== tmp.getMonth()) {
      w = i + 1
      break;
    }
  }

  return  mv === m ? w : -w
}
export const calcLastMonthFirstDay = v => {
  return calcMonthFirstDay(calcLastMonth(v))
}
export const calcMonthFirstDay = v => {
  return new Date(v.getFullYear(), v.getMonth(), 1)
}
export const calcLastMonth = (v) => {
  return calcMonth(v, -1)
}
export const calcNextMonth = (v) => {
  return calcMonth(v, 1)
}
export const calcMonth = (v, num) => {
  const ret = new Date(v)
  let y = takeYear(v)
  let m = takeMonth(v)
  
  const mos = parseInt(num % 12)
  const yos = parseInt(num / 12)
  y += yos
  m += mos

  if (m > 12) {
    y += 1
    m -= 12
  } else if (m < 1) {
    y -= 1
    m += 12
  }

  ret.setMonth(m - 1)
  ret.setFullYear(y)
  return ret
}
export const calcDay = (v, num) => {
  const vtime = v.getTime()
  return new Date(vtime + (num * 24 * 60 * 60 * 1000))
}

export function format (date, fmt) {
  const fmtStr = fmt || FMT_DATE_TIME
  const res = fmtStr.match(/\[(\s*|\S*)\]/)
  let ret = fmtStr
  if (res && res[0]) {
    ret = fmtStr.replace(res[0], '[]')
  }
  
  handlers.forEach(fun => { 
    ret = fun.call(null, date, ret)
  })

  if (res && res[0]) {
    ret = ret.replace('[]', 'W')
  }
  return ret
}

const takeFromString = (str, fmt, reg) => {
  const res = fmt.match(reg)
  let ret = undefined
  if (res) {
    ret = str.substring(res.index, res.index + res[0].length)
  }
  return ret
}

const createNCharString = (char, num) => {
  let ret = ''
  for (let i = num; i > 0; i--) {
    ret += char
  }
  return ret
}

const parseWeekOfMonth = (str, fmt) => {
  let fmtStr = fmt
  const res = fmt.match(/\[(\s+|\S+)\]/)
  if (res) {
    fmtStr = fmt.replace(res[0], createNCharString('[', res[0].length - 2))
  }
  const y = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_YEAR))
  const m = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_MONTH))
  const w = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_WEEK_MONTH))
  if (y && m && w) {
    let ret = new Date(y, m - 1, 1) 
    const retw = ret.getDay()
    ret = calcDay(ret, TABLE_NEXT_MONDAY[retw] + 7 * (w-1))
    return ret
  } else {
    return undefined
  }
}

const parseWeekOfYear = (str, fmt) => {
  let fmtStr = fmt
  const res = fmt.match(/\[(\s+|\S+)\]/)
  if (res) {
    fmtStr = fmt.replace(res[0], createNCharString('[', res[0].length - 2))
  }
  const y = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_YEAR))
  const w = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_WEEK_YEAR))
  if (y && w) {
    let ret = new Date(y, 0, 1)
    const retw = ret.getDay()
    ret = calcDay(ret, TABLE_NEXT_MONDAY[retw] + 7 * (w - 1))
    return ret
  } else {
    return undefined
  }
}

const parseOther = (str, fmt) => {
  let fmtStr = fmt
  const res = fmt.match(/\[(\s+|\S+)\]/)
  if (res) {
    fmtStr = fmt.replace(res[0], createNCharString('[', res[0].length - 2))
  }
  const y = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_YEAR))
  const m = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_MONTH))
  const d = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_DAY))
  const h = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_HOUR))
  const mu = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_MSECOND))
  const s = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_SECOND))
  const ms = parseInt(takeFromString(str, fmtStr, REG_EXP_DATE_FMT_MSECOND))
  const ret = new Date()
  if (y) ret.setFullYear(y)
  if (m) ret.setMonth(m - 1)
  if (d) ret.setDate(d)
  if (h) ret.setHours(h)
  if (mu) ret.setMinutes(mu)
  if (s) ret.setSeconds(s)
  if (ms) ret.setMilliseconds(ms)
  return ret
}

const fmtToRegExp = (fmt) => {
  let regStr = fmt
    ;[REG_EXP_DATE_FMT_DAY, REG_EXP_DATE_FMT_YEAR, REG_EXP_DATE_FMT_MONTH, 
      REG_EXP_DATE_FMT_HOUR, REG_EXP_DATE_FMT_MINUE, REG_EXP_DATE_FMT_SECOND,
      REG_EXP_DATE_FMT_MSECOND, REG_EXP_DATE_FMT_WEEK_YEAR, REG_EXP_DATE_FMT_WEEK_MONTH
    ].forEach(it => { 
      const rey = regStr.match(it)
      if (rey) {
        regStr = regStr.replace(rey[0], `(\\d{${rey[0].length}})`)
        console.log(fmt, regStr)
      }
    })
  console.log(fmt, regStr)
  const res = regStr.match(/\[(\s+|\S+)\]/)
  if (res) {
    regStr = regStr.replace(res[0], res[0].substring(1, res[0].length - 1))
  }
  return new RegExp(regStr)
}

const judgeFmts = (str, fmts) => {
  let ret = ''
  for (let i = 0; i < fmts.length; i++) {
    const res = str.match(fmtToRegExp(fmts[i]))
    if (res && res[0].length === str.length) {
      ret = fmts[i]
      break
    }
  }
  return ret
}

export function parse (str, fmts) {
  let fmt = fmts
  if (isArray(fmts)) {
    fmt = judgeFmts(str, fmts)
  }

  const fmtStr = fmt || FMT_DATE_TIME
  let res = fmtStr.match(REG_EXP_DATE_FMT_WEEK_MONTH)
  if (res) {
    return parseWeekOfMonth(str, fmtStr)
  } else {
    res = fmtStr.match(REG_EXP_DATE_FMT_WEEK_YEAR)
    if (res) {
      return parseWeekOfYear(str, fmtStr)
    }
  }
  return parseOther(str, fmtStr)
}


export default {
  takeYear,
  takeMonth,
  takeDayOfMonth,
  takeHours,
  takeSeconds,
  takeMinutes,
  takeMilliseconds,
  takeWeekOfYear,
  takeWeekOfMonth,

  calcLastMonthFirstDay,
  calcMonthFirstDay,
  calcLastMonth,
  calcNextMonth,

  calcMonth,
  calcDay,
  format,
  parse
}